#!/usr/bin/python
from psutil import disk_partitions as _disk_partitions, disk_usage as _disk_usage

def _pretty_size(size):
    size_strs = ['B', 'KiB', 'MiB', 'GiB', 'TiB']
    last_size = size
    fract_size = size
    num_divs = 0

    while size > 1:
        fract_size = last_size
        last_size = size
        size /= 1024.0
        num_divs += 1

    num_divs -= 1
    fraction = fract_size / 1024.0
    pretty = "%.2f" % fraction
    pretty = pretty + size_strs[num_divs]
    return pretty

def getpart():
    partlist = _disk_partitions()
    partlist.sort()
    partdict={}
    for i in partlist:
        dev = i.device
        mount = i.mountpoint
        fstype = i.fstype
        total = _pretty_size(_disk_usage(mount).total)
        partdict[i.device]={'mount':mount,'fstype':fstype,'total':total}
    return partdict



#!/usr/bin/python
import subprocess,sys

def parse_dmi(content,hwtype):
    info = []
    lines = iter(content)
#    lines = iter(content.strip().splitlines())
    while True:
        try:
            line = next(lines)
        except StopIteration:
            break
        if line.startswith('Handle 0x'):
            info.append((hwtype, _parse_handle_section(lines)))
    return info

def _parse_handle_section(lines):
    data = {
        '_title': next(lines).rstrip(),
        }

    for line in lines:
        line = line.rstrip()
        if line.startswith('\t\t'):
            if isinstance(data[k], list):
                data[k].append(line.lstrip())
        elif line.startswith('\t'):
            k, v = [i.strip() for i in line.lstrip().split(':', 1)]
            if v:
                data[k] = v
            else:
                data[k] = []
        else:
            break
    return data

def info(shell):
    try:
        output = subprocess.check_output(
        'PATH=$PATH:/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin '
        'sudo ' + shell, shell=True)
    except Exception as e:
        print(e)
        if str(e).find("command not found") == -1:
            print("please install dmidecode")
            print("e.g. sudo yum install dmidecode")

        sys.exit(1)
    return output.decode('utf-8').strip().splitlines()
#    return output.decode('utf-8').strip().split('\n')


def getserver():
    hwvendor = info("dmidecode -t system|grep -C2 Version|awk -F:\  '{print $2}'")
    hwclass = info("dmidecode -t chassis|grep -E 'Type|Height'|awk -F:\  '{print $2}'")
    serverdict = {'Server':{'Vendor':hwvendor[0],'Type':hwvendor[1],'Version':hwvendor[2],'SN':hwvendor[3],'UUID':hwvendor[4]},'class':{'Server Type':hwclass[0],'Height':hwclass[1]}}
    return serverdict


def getcpu():
    hwcpu = parse_dmi(info("dmidecode -t processor"),"cpu")
    cpudict = {}
    hascpu = 0
    for i in hwcpu:
        cnum = i[1]['Socket Designation']
        if i[1]['Status'] == 'Populated, Enabled':
            hascpu += 1
            ctype = i[1]['Version']
            ccc = i[1]['Core Count']
            try:
                ctc = i[1]['Thread Count']
            except:
                ctc = False
            ccspeed = i[1]['Current Speed']
            cpudict[cnum] = {'Type':ctype,'Core Count':ccc,'Thread Count':ctc,'Current Speed':ccspeed}
        else:
            cpudict[cnum] = {'Status':i[1]['Status']}

    hwctotal = len(hwcpu)
    hwcsocket = hwcpu[0][1]['Upgrade']
    hwcmspeed = hwcpu[0][1]['Max Speed']
    cpudict['cpusocket'] = {'Socket Count':hwctotal,'CPU Count':hascpu,'Socket Type':hwcsocket,'Max Speed':hwcmspeed}
    return cpudict


def getmem():
    hwmem = parse_dmi(info("dmidecode -t memory"),"memory")
    memdict = {}
    hasmem = 0
    for i in hwmem:
        if i[1]['_title'] == 'Memory Device':
            memnum = 'MEM-' + i[1]['Locator']
            if i[1]['Size'] != 'No Module Installed':
                hasmem +=1
                msize = i[1]['Size']
                mtype = i[1]['Type']
                mmspeed = i[1]['Speed']
                mcspeed = i[1]['Configured Clock Speed']
                mfac = i[1]['Manufacturer']
                mpn = i[1]['Part Number']
                memdict[memnum] = {'Size':msize,'Type':mtype,'Speed':mmspeed,'Current Speed':mcspeed,'Vendor':mfac,'Model':mpn}
            else:
                memdict[memnum] = {'Status':"No Module Installed"}
        elif i[1]['_title'] == 'Physical Memory Array':
            hwmtotalnum = i[1]['Number Of Devices']
            hwmtotalsize = i[1]['Maximum Capacity']
            hwmerr = i[1]['Error Correction Type']
            memdict['memsocket'] = {'Mem Count':hwmtotalnum,'Max Size':hwmtotalsize,'Error Correction Type':hwmerr}
    memdict['memsocket']['Current Mem Count'] = hasmem
    return memdict



#!/usr/bin/python
from __future__ import print_function
from psutil import net_if_addrs,net_if_stats
import os
import re
import sys
import errno
from subprocess import Popen,PIPE
p = Popen(["lspci"], stdout=PIPE)
err = p.wait()
if err:
    print ("Error running lspci")
    sys.exit()
pcidata = p.stdout.read().decode('utf-8')

def _find_device(data, pciid):
    id = re.escape(pciid)
    m = re.search("^" + id + "\s(.*)$", data, re.MULTILINE)
    return m.group(1)
def getnet():
    netdict = {}
    for k, v in net_if_addrs().items():
        if not (k.startswith('tap') or k.startswith('vir') or k.startswith('br') or k.startswith('lo')):
            for item in v:
                mac = item[1]
                if ':' in mac and len(mac)==17:
                    netdir = os.readlink(os.path.join("/sys/class/net", k))
                    m = re.match(".+/\d+:(\w+:\w+\.\w)/net/\s*", netdir)
                    pciid = m.group(1)
                    host = _find_device(pcidata, pciid)
                    isup = net_if_stats()[k].isup
                    if isup == True:
                        speed = str(net_if_stats()[k].speed) + 'Mbps'
                    else:
                        speed = False
                    netdict[k] = {'host':host,'mac':mac,'isup':isup,'speed':speed}
    return netdict



#!/usr/bin/python
from hwpy import _hwnet, _hwpart, _hwdisk, _dmide
net = _hwnet.getnet()
part = _hwpart.getpart()
disk = _hwdisk.getdisk()
cpu = _dmide.getcpu()
host = _dmide.getserver()
mem = _dmide.getmem()
hwlist = [host,cpu,mem,disk,part,net]
def main():
    for i in hwlist:
        items = i.keys()
        itemslist = sorted(items)
        for m in itemslist:
            print (m + ':')
            for ik,iv in i[m].items():
                print ("\t" + ik + ':' + str(iv))



#!/usr/bin/python
from __future__ import print_function
import os
import re
import sys
import errno
from subprocess import Popen,PIPE

def find_device(data, pciid):
    id = re.escape(pciid)
    m = re.search("^" + id + "\s(.*)$", data, re.MULTILINE)
    return m.group(1)

def pretty_size(size):
    size_strs = ['B', 'KiB', 'MiB', 'GiB', 'TiB']
    last_size = size
    fract_size = size
    num_divs = 0

    while size > 1:
        fract_size = last_size
        last_size = size
        size /= 1024
        num_divs += 1

    num_divs -= 1
    fraction = fract_size / 1024
    pretty = "%.2f" % fraction
    pretty = pretty + size_strs[num_divs]
    return pretty

def virtual_device(path):
    for dir in os.listdir(path):
        if re.search("device", dir):
            return 0
    return 1

class Device:
    def __init__(self):
        self.sectorsize = ""
        self.sectors = ""
        self.rotational = ""
        self.sysdir = ""
        self.host = ""
        self.model = ""
        self.vendor = ""
        self.holders = []
        self.diskname = ""
        self.partitions = []
        self.removable = ""
        self.start = ""
        self.discard = ""
        self.sysfs_no_links = 0

    def populate_model(self):
        try:
            f = open(self.sysdir + "/device/model")
            self.model = f.read().rstrip()
            f.close()
        except IOError:
            # do nothing
            pass

    def populate_vendor(self):
        try:
            f = open(self.sysdir + "/device/vendor")
            self.vendor = f.read().rstrip()
            f.close()
        except IOError:
            #do nothing
            pass

    def populate_sectors(self):
        try:
            f = open(self.sysdir + "/size")
            self.sectors = f.read().rstrip()
            f.close()
        except IOError:
            self.sectors = 0

    def populate_sector_size(self):
        try:
            f = open(self.sysdir + "/queue/hw_sector_size")
            self.sectorsize = f.read().rstrip()
            f.close()
        except IOError:
            # if this sysfs doesnt show us sectorsize then just assume 512
            self.sectorsize = "512"

    def populate_rotational(self):
        try:
            f = open(self.sysdir + "/queue/rotational")
            rotation = f.read().rstrip()
            f.close()
        except IOError:
            self.rotational = "Could not determine rotational"
            return
        if rotation == "1":
            self.rotational = "Spinning disk"
        else:
            self.rotational = "SSD"

    def populate_host(self, pcidata):
        if self.sysfs_no_links == 1:
            try:
                sysdir = os.readlink(os.path.join(self.sysdir, "device"))
            except:
                pass
        else:
            sysdir = self.sysdir
#        m = re.match(".+/\d+:(\w+:\w+\.\w)/host\d+/\s*", sysdir)
        m = re.match(".+/\d+:(\w+:\w+\.\w)/[a-z]+\d+/\s*", sysdir)
        if m:
            pciid = m.group(1)
            self.host = find_device(pcidata, pciid)
        else:
            self.host = ""

    def populate_diskname(self):
        m = re.match(".*/(.+)$", self.sysdir)
        self.diskname = m.group(1)

    def populate_holders(self):
        for dir in os.listdir(self.sysdir + "/holders"):
            if re.search("^dm-.*", dir):
                try:
                    f = open(self.sysdir + "/holders/" + dir + "/dm/name")
                    name = f.read().rstrip()
                    f.close()
                    self.holders.append(name)
                except IOError:
                    self.holders.append(dir)
            else:
                self.holders.append(dir)

    def populate_discard(self):
        try:
            f = open(self.sysdir + "/queue/discard_granularity")
            discard = f.read().rstrip()
            f.close()
            if discard == "0":
                self.discard = "No"
            else:
                self.discard = "Yes"
        except IOError:
            self.discard = "No"

    def populate_start(self):
        try:
            f = open(self.sysdir + "/start")
            self.start = f.read().rstrip()
            f.close()
        except IOError:
            pass

    def populate_partitions(self):
        for dir in os.listdir(self.sysdir):
            m = re.search("(" + self.diskname + "\d+)", dir)
            if m:
                partname = m.group(1)
                part = Device()
                part.sysdir = self.sysdir + "/" + partname
                part.populate_part_info()
                self.partitions.append(part)

    def populate_part_info(self):
        """ Only call this if we are a partition """
        self.populate_diskname()
        self.populate_holders()
        self.populate_sectors()
        self.populate_start()

    def populate_removable(self):
        try:
            f = open(self.sysdir + "/removable")
            remove = f.read().rstrip()
            f.close()
            if remove == "1":
                self.removable = "Yes"
            else:
                self.removable = "No"
        except IOError:
            self.removable = "No"

    def populate_all(self, pcidata):
        self.populate_diskname()
        self.populate_holders()
        self.populate_partitions()
        self.populate_removable()
        self.populate_model()
        self.populate_vendor()
        self.populate_sectors()
        self.populate_sector_size()
        self.populate_rotational()
        self.populate_discard()
        self.populate_host(pcidata)

def getdisk():
    p = Popen(["lspci"], stdout=PIPE)
    err = p.wait()
    if err:
        print ("Error running lspci")
        sys.exit()
    pcidata = p.stdout.read().decode('utf-8')

    sysfs_no_links = 0
    devices = []

    if len(sys.argv) > 1:
        m = re.match("/dev/(\D+)\d*", sys.argv[1])
        if m:
            block = m.group(1)
        else:
            block = sys.argv[1]

        try:
            path = os.readlink(os.path.join("/sys/block/", block))
        except OSError as e:
            if e.errno == errno.EINVAL:
                path = block
            else:
                print ("Invalid device name " + block)
                sys.exit()
        d = Device()
        d.sysdir = os.path.join("/sys/block", path)
        d.populate_all(pcidata)
        devices.append(d)
    else:
        for block in os.listdir("/sys/block"):
            try:
                if sysfs_no_links == 0:
                    path = os.readlink(os.path.join("/sys/block/", block))
                else:
                    path = block
            except OSError as e:
                if e.errno == errno.EINVAL:
                    path = block
                    sysfs_no_links = 1
                else:
                    continue
            if re.search("virtual", path):
                continue
            if sysfs_no_links == 1:
                sysdir = os.path.join("/sys/block", path)
                if virtual_device(sysdir) == 1:
                    continue
            d = Device()
            d.sysdir = os.path.join("/sys/block", path)
            d.sysfs_no_links = sysfs_no_links
            d.populate_all(pcidata)
            devices.append(d)
    diskdict = {}
    devices.sort()
    for d in devices:
        diskdict[d.diskname] = []
        size = float(d.sectors) * float(d.sectorsize)
        allpretty = pretty_size(size)
        diskdict[d.diskname] = {'host':d.host,'vendor':d.vendor,'model':d.model,'size':allpretty,'type':d.rotational}
    return diskdict



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: hwinfo
# Author: Stars Liao
# Mail: starsliaop@163.com
# Created Time:  2018-12-24 19:17:34
#############################################

from setuptools import setup, find_packages

setup(
    name = "hwpy",
    version = "0.1.0",
    keywords = ("pip", "hwinfo","Hardware"),
    description = "hwpy",
    long_description = "Show Linux Hardware Info",
    license = "MIT Licence",

    url = "https://github.com/starsliao/hwpy",
    author = "Stars Liao",
    author_email = "starsliao@163.com",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = ["psutil"],
    classifiers=[
        'Intended Audience :: System Administrators',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Utilities',
        ],
)
